@@ -2,10 +2,8 @@
 
 use strict;
 
-# benchmark Euklid against binarybgcd() algorithmn
-use lib '../lib';
+#use lib '../lib';
 
-#use Math::BigInt;
 use Math::BigInt lib => 'BitVect';
 use Math::BigFloat;
 use Math::Big;
@@ -14,17 +12,17 @@ use Benchmark;
 
 my $digits = shift || 100;
 my $fibo = shift || 100;
-my $c = 400;
+my $c = -4;
 
 # takes longest when $x and $y are consecutive fibonacci numbers
 my $x = Math::Big::fibonacci($fibo);
 my $y = Math::Big::fibonacci($fibo+1);
 my $z; my $u = ''; my $v = '';
-while (length($u))
+while (length($u) < $digits)
   {
   $u .= int(rand(10000));
   }
-while (length($u))
+while (length($v) < $digits)
   {
   $v .= int(rand(10000));
   }
@@ -38,7 +36,7 @@ timethese ( $c,
   {
   bgcd_fibu => sub { $z = $x->bgcd($y); },
   } ) ;
-timethese ( $c*15, 
+timethese ( $c, 
   {
   bgcd_rand => sub { $z = $u->bgcd($v); }
   } ) ;
