@@ -5,8 +5,6 @@ BEGIN { plan tests => 7; }
 
 BEGIN { unshift @INC, '../lib'; }	# comment out to use old module
 use strict;
-#use Math::BigInt;
-#use Math::BigInt qw/:constant/;
 use Math::BigInt qw/lib BitVect :constant/;
 
 my ($x,$y,$z);
@@ -17,9 +15,13 @@ my $two = Math::BigInt->new(2);
  $x = $two ** 756839; $x--;   ok(1,1);  #ok (len($x),227832);
 
 # some twin primes (first in list at 03/2001)
-$x = ($two ** 80025) * 665551035; $x++; $y = $x-2; ok (1,1); #ok (len($x),24099);
-$x = ($two ** 66443) * 1693965; $x++; $y = $x-2;   ok (1,1); #ok (len($x),20008);
-$x = ($two ** 64955) * 83475759; $x++; $y = $x-2;  ok (1,1); #ok (len($x),19562);
-# ...
-$x = ($two ** 38880) * 242206083; $x++; $y = $x-2; ok (1,1); #ok (len($x),11713);
+$x = ($two ** 80025) * 665551035; $x++; $y = $x-2; ok (1,1);
+$x = ($two ** 66443) * 1693965; $x++; $y = $x-2;   ok (1,1);
+$x = ($two ** 64955) * 83475759; $x++; $y = $x-2;  ok (1,1);
+$x = ($two ** 38880) * 242206083; $x++; $y = $x-2; ok (1,1);
+
+sub len 
+  {
+  my $x = shift; length("$x");
+  }
 
